﻿/**
* @file StovePCDefine.h
* @brief StovePCDefine 헤더파일
* @details StovePCSDK 사용하는 enum, struct를 선언합니다.
*/
#ifndef STOVEPCDEFINE_H_
#define STOVEPCDEFINE_H_

/*!
*  @struct StovePCResult
*  @brief 스토브에서 정의한 API 호출결과에 대한 enum 입니다.
*  @brief API 호출시 API 호출 후 결과 상태에 따라 반환됩니다.
*/
enum StovePCResult
{
    /// 정상 
    STOVE_PC_NO_ERROR = 0,

    // Config & Env Error  50~99
    /// 설정된 환경값이 유효하지 않음
    STOVE_PC_INVALID_CONFIG             = 50,

    /// 설정된 콜백이 유효하지 않음 
    STOVE_PC_INVALID_CALLBACK           = 51,

    /// 설정된 로그레벨의 값이 유효하지 않음
    STOVE_PC_INVALID_LOG_LEVEL          = 52,

    /// 설정된 로그경로가 유효하지 않음 
    STOVE_PC_INVALID_LOG_PATH           = 53,

    /// 서버로부터 환경정보를 받지 못함 
    STOVE_PC_INVALID_ENV                = 54,

    //Core Error          100 ~149
    /// StovePCSDK 가 초기화 되지 않음  
    STOVE_PC_CORE_NOT_INITIALIZE            = 100,

    /// StovePCSDK 가 이미 초기화 됨 
    STOVE_PC_CORE_ALREADY_INITIALIZE        = 101,

    /// 내부의 모든 Task가 비어 있음
    STOVE_PC_CORE_TASKS_ALL_EMPTY           = 102,

    /// 콜백 호출시 콜백의 주소가 없음 
    STOVE_PC_CORE_CALLBACK_EXCUTE_NULLPTR   = 103,

    //SGUP Error 150 ~ 199
    ///  SDK 내부의 SGUP TASK가 초기화 되지 않음 
    STOVE_PC_SGUP_NOT_INITIALIZE            = 150,

    /// SGUP Task가 유효하지 않음
    STOVE_PC_SGUP_TASK_EMPTY                = 151,

    /// SGUP Task에서 외부에러가 발생함[deprecated]
    STOVE_PC_SGUP_EXTERNAL_ERROR            = 152,

    /// SGUP OnToken 콜백이 없음 
    STOVE_PC_SGUP_ONTOKEN_EMPTY             = 153,

    /// SGUP OnUser 콜백이 없음  
    STOVE_PC_SGUP_ONUSER_EMPTY              = 154,

    /// SGUP Token이 유효하지 않음 
    STOVE_PC_SGUP_TOKEN_INVALID             = 155,

    //OwnerShip Error 200 ~ 249
    /// Ownership Task가 초기화 되지 않음[deprecated]
    STOVE_PC_OWNERSHIP_NOT_INITIALIZE       = 200,

    /// Ownership 조회시 토큰이 유효하지 않음
    STOVE_PC_OWNERSHIP_TOKEN_INVALID        = 201,

    /// Ownership Task가 비어있음 
    STOVE_PC_OWNERSHIP_TASK_EMPTY           = 202,

    /// Ownership OnOwnership 콜백이 비어 있음 
    STOVE_PC_OWNERSHIP_ONOWNERSHIP_EMPTY    = 203,

    /// Ownership 조회시 HTTP Error 발생 
    STOVE_PC_OWNERSHIP_HTTP_ERROR           = 204,

    /// Ownership 조회 후 응답코드 비정상
    STOVE_PC_OWNERSHIP_RESPONSE_ERROR       = 205,

    //Stats Error 250 ~ 299
    /// Stats 기능 호출시 파라미터가 유효하지 않음
    STOVE_PC_STATS_PARAM_INVALID            = 250,

    /// Stats 조회시 토큰이 유효하지 않음
    STOVE_PC_STATS_TOKEN_INVALID            = 251,

    /// Stats Task가 비어있음 
    STOVE_PC_STATS_TASK_EMPTY               = 252,

    /// Stats 콜백이 비어 있음 
    STOVE_PC_STATS_ONSTAT_EMPTY             = 253,

    /// Stats 콜백이 비어 있음 
    STOVE_PC_STATS_ONSETSTAT_EMPTY          = 254,

    /// Stats 조회시 HTTP Error 발생 
    STOVE_PC_STATS_HTTP_ERROR               = 255,

    /// Stats 조회 후 응답코드 비정상
    STOVE_PC_STATS_RESPONSE_ERROR           = 256,

    //Achievement Error 300 ~ 349
    /// Achievement 기능 호출시 파라미터가 유효하지 않음
    STOVE_PC_ACHIEVEMENT_PARAM_INVALID          = 300,

    /// Achievement 조회시 토큰이 유효하지 않음
    STOVE_PC_ACHIEVEMENT_TOKEN_INVALID          = 301,

    /// Achievement Task가 비어있음 
    STOVE_PC_ACHIEVEMENT_TASK_EMPTY             = 302,

    /// Achievement 콜백이 비어 있음 
    STOVE_PC_ACHIEVEMENT_ONACHIEVEMENT_EMPTY    = 303,

    /// Achievement 콜백이 비어 있음 
    STOVE_PC_ACHIEVEMENT_ONALLACHIEVEMENT_EMPTY = 304,

    /// Achievement 조회시 HTTP Error 발생 
    STOVE_PC_ACHIEVEMENT_HTTP_ERROR             = 305,

    /// Achievement 조회 후 응답코드 비정상
    STOVE_PC_ACHIEVEMENT_RESPONSE_ERROR         = 306,

    //Leaderboard Error 350 ~ 399
    /// Leaderboard 기능 호출시 파라미터가 유효하지 않음
    STOVE_PC_LEADERBOARD_PARAM_INVALID          = 350,

    /// Leaderboard 조회시 토큰이 유효하지 않음
    STOVE_PC_LEADERBOARD_TOKEN_INVALID          = 351,

    /// Leaderboard Task가 비어있음 
    STOVE_PC_LEADERBOARD_TASK_EMPTY             = 352,

    /// Leaderboard 콜백이 비어 있음 
    STOVE_PC_LEADERBOARD_ONRANK_EMPTY           = 353,

    /// Leaderboard 조회시 HTTP Error 발생 
    STOVE_PC_LEADERBOARD_HTTP_ERROR             = 354,

    /// Leaderboard 조회 후 응답코드 비정상
    STOVE_PC_LEADERBOARD_RESPONSE_ERROR         = 355,

    //Member Error 400 ~ 449
    /// Member 조회시 토큰이 유효하지 않음
    STOVE_PC_MEMBER_TOKEN_INVALID               = 400,

    /// Member Task가 비어있음 
    STOVE_PC_MEMBER_TASK_EMPTY                  = 401,

    /// Member 조회시 HTTP Error 발생 
    STOVE_PC_MEMBER_HTTP_ERROR                  = 402,

    /// Member 조회 후 응답코드 비정상
    STOVE_PC_MEMBER_RESPONSE_ERROR              = 403,

    // Exception 900~
    /// StoveSDK  내부에서 Exception 발생
    STOVE_PC_UNMANAGED_EXCEPTION                = 900,

    /// StoveSDK 외부에서 Exception 발생 
    STOVE_PC_MANAGED_EXCEPTION                  = 901
};

/*!
*  @struct StovePCLogLevel
*  @brief 로그레벨 설정을 위한 enum 타입입니다.
*/
enum StovePCLogLevel
{
    /// 로그 파일을 생성하지 않음
    STOVE_PC_LOG_LEVEL_NONE = 1,

    /// 로그레벨 Error 설정
    STOVE_PC_LOG_LEVEL_ERROR = 2,

    /// 로그레벨 Info 설정
    STOVE_PC_LOG_LEVEL_INFO = 3,

    /// 로그레벨 Debug 설정
    STOVE_PC_LOG_LEVEL_DEBUG = 4
};

/*!
*  @struct StovePCInitState
*  @brief StovePC_Init을 호출한 후의 sdk 내부 상태를 나타냅니다.
*/
enum StovePCInitState
{
    /// StovePC_Init이 호출되지 않은 상태(초깃값)
    STOVE_PC_INIT_NONE      = 0,

    /// StovePC_Init이 호출된 후, 초기화 진행 중
    STOVE_PC_INIT_PENDING   = 1,

    /// StovePC_Init이 호출된 후, 초기화 실패.(OnError 콜백이 호출됩니다.)
    STOVE_PC_INIT_FAIL      = 2,

    /// StovePC_Init이 호출된 후, 초기화 완료.(OnInitComplete 콜백이 호출됩니다.)
    STOVE_PC_INIT_COMPLETE  = 3,
};

/*!
*  @struct StovePCFunctionType
*  @brief OnError 콜백에서 어떤 함수가 호출됐는지 구분하기 위해 사용되는 enum 타입입니다.
*/
enum StovePCFunctionType
{
    /// StovePC_Init 함수
    STOVE_PC_INIT                   = 0,

    /// StovePC_GetToken 함수
    STOVE_PC_GET_TOKEN              = 1,

    /// StovePC_GetUser 함수
    STOVE_PC_GET_USER               = 2,

    /// StovePC_GetOwnership 함수
    STOVE_PC_GET_OWNERSHIP          = 3,

    /// StovePC_GetStat 함수
    STOVE_PC_GET_STAT               = 4,

    /// StovePC_SetStat 함수
    STOVE_PC_SET_STAT               = 5,

    /// StovePC_GetAchievement 함수
    STOVE_PC_GET_ACHIEVEMENT        = 6,

    /// StovePC_GetAllAchievement 함수
    STOVE_PC_GET_ALL_ACHIEVEMENT    = 7,

    /// StovePC_GetRank 함수
    STOVE_PC_GET_RANK               = 8,
};

/*!
*  @struct StovePCError
*  @brief OnError Callback 호출시 전달받는 구조체입니다.
*
*/
#pragma pack(push, 4)
struct StovePCError
{
    /// 호출된 함수를 나타내는 enum 값
    StovePCFunctionType functionType;

    /// Error 에 대한 유형 결과값 
    StovePCResult result;

    /// Error가 발생한 이유에 대한 상세 메세지
    char* message;

    /// 외부에서 발생한 Error 값 (HTTP error or API result code)
    int externalError;
};
#pragma pack(pop)

/*!
*  @struct StovePCConfig
*  @brief SDK Int 호출시 설정되는 환경 값입니다.
*/
struct StovePCConfig
{
    /// StovePCSDK의  Stove 환경: "QA", "SANDBOX", "LIVE"(empty string이면 "SANDBOX" 환경)
    char* env;

    /// 스토브에서 발급 받은 Application Key
    char* appKey;

    /// 스토브에서 발급 받은 Secret Key
    char* secretKey;

    /// 스토브에서 사용할 game ID
    wchar_t* gameId;

    /// SDK 로그 레벨 설정 
    StovePCLogLevel logLevel;

    /// SDK 로그경로 설정 (empty string이면 현재 실행 경로에 생성 )
    wchar_t* logPath;
};

/*!
*  @struct StovePCToken
*  @brief StovePC_GetToken API 호출시 응답받는 콜백의 Token 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCToken
{
    /// 런처에서 발행한 토큰 값
    char* accessToken;
};
#pragma pack(pop)

/*!
*  @struct StovePCUser
*  @brief StovePC_GetUser API 호출시 응답받는 콜백의 UserInfo 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCUser
{
    /// 런처에서 로그인한 유저의 스토브 회원번호 
    unsigned __int64 memberNo;

    /// 런처에서 로그인한 유저의 스토브 닉네임 
    wchar_t* nickname;

    /// 런처에서 로그인한 유저의 gameUserId(GameID + MemeberNo)
    char* gameUserId; 
};
#pragma pack(pop)

/*!
*  @struct StovePCOwnership
*  @brief StovePC_GetOwnership 관련 API 호출시 응답받는 콜백의 OwnerShip 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCOwnership
{
    /// Ownership 게임 아이디 
    wchar_t* gameId;

    /// Ownership 게임 코드 
    int gameCode;

    /// Ownership 코드 
    int ownershipCode;

    /// Ownership 구매날짜 
    unsigned __int64 purchaseDate;

    /// Ownership 멤버넘버 
    unsigned __int64 memberNo;
};
#pragma pack(pop)

/*!
*  @struct StovePCStatFullId
*  @brief StovePC_GetStat 관련 API 호출시 응답받는 콜백의 StatFullId 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCStatFullId
{
    /// Stats 게임 아이디 
    wchar_t* gameId;

    /// Stat 아이디,, [A-Z0-9\\-_]*$ 형식(영문 대문자, 숫자, underscore, hyphen)
    char* statId;
};
#pragma pack(pop)

/*!
*  @struct StovePCStat
*  @brief StovePC_GetStat 관련 API 호출시 응답받는 콜백의 Stat 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCStat
{
    /// Stat 전체아이디
    StovePCStatFullId statFullId;

    /// Stat 현재값
    int currentValue;

    /// 멤버넘버 
    unsigned __int64 memberNo;

    /// Stat 최종 업데이트 시간(epoch time milliseconds)
    unsigned __int64 updatedAt;
};
#pragma pack(pop)

/*!
*  @struct StovePCStatValue
*  @brief StovePC_SetStat 관련 API 호출시 응답받는 콜백의 StatValue 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCStatValue
{
    /// Stat 현재값
    int currentValue;

	/// Stat 값의 변경 여부
	bool updated;

    /// 오류 메시지 (eg. integer overflow) 
    char* errorMessage;
};
#pragma pack(pop)

/*!
*  @struct StovePCAchievementCondition
*  @brief StovePC_GetAchievement, StovePC_GetAllAchievement 관련 API 호출시 응답받는 콜백의 AchievementCondition 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCAchievementCondition
{
    /// Achievement 달성 목표값
    int goalValue;

    /// 값 처리 방법 (DELTA_SUM: 누적 데이터 합계, REPLACE: 기존 데이터 교체, MAX : 큰값으로 대체, MIN : 작은값으로 대체)
    char* valueOperation;

    /// Achievement 목표값과 사용자 업적 값의 비교 연산 방식 ( GTE: >=, LTE: <= )
    char* type;
};
#pragma pack(pop)

/*!
*  @struct StovePCAchievement
*  @brief StovePC_GetAchievement, StovePC_GetAllAchievement 관련 API 호출시 응답받는 콜백의 Achievement 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCAchievement
{
    /// Achievement 아이디
    char* achievementId;

    /// Achievement 이름
    wchar_t* name;

    /// Achievement 설명
    wchar_t* description;

    /// 기본 이미지 URL
    wchar_t* defaultImage;

    /// 달성 이미지 URL
    wchar_t* achievedImage;

    /// Achievement 조건
    StovePCAchievementCondition condition;

    /// Achievement 현재값
    int value;

    /// Achievement 상태 (ONGOING : 진행중, ACHIEVED: 업적 달성)
    char* status;
};
#pragma pack(pop)

/*!
*  @struct StovePCRank
*  @brief StovePC_GetRank 관련 API 호출시 응답받는 콜백의 Rank 구조체입니다.
*/
#pragma pack(push, 4)
struct StovePCRank
{
    /// 멤버넘버 
    unsigned __int64 memberNo;

    /// 점수
    int score;

    /// 순위
    unsigned int rank;

    /// 닉네임
    wchar_t* nickname;

    /// 프로필 이미지 URL
    wchar_t* profileImage;
};
#pragma pack(pop)

/*!
*  @struct StovePCCallback
*  @brief Stove API 호출시 응답받는 콜백을 설정하는 구조체입니다.
*/
struct StovePCCallback
{
    /// StovePCSDK 에서 에러발생시 호출되는 콜백 함수 포인터 
    void(*OnError)(const StovePCError error);

    /// Init API 호출시 응답받는 콜백 함수 포인터
    void(*OnInitComplete)();

    /// GetToken API 호출시 응답받는 콜백 함수 포인터 
    void(*OnToken)(const StovePCToken token);

    /// GetUser API 호출시 응답받는 콜백 함수 포인터
    void(*OnUser)(const StovePCUser user);

    /// GetOwnership API 호출시 응답받는 콜백 함수 포인터
    void(*OnOwnership)(int size, StovePCOwnership* ownership);

    /// GetStat API 호출시 응답받는 콜백 함수 포인터
    void(*OnStat)(const StovePCStat stat);

    /// SetStat API 호출시 응답받는 콜백 함수 포인터
    void(*OnSetStat)(const StovePCStatValue statValue);

    /// GetAchievment API 호출시 응답받는 콜백 함수 포인터
    void(*OnAchievement)(const StovePCAchievement achievement);

    /// GetAchievments API 호출시 응답받는 콜백 함수 포인터
    void(*OnAllAchievement)(int size, StovePCAchievement* achievement);

    /// GetRank API 호출시 응답받는 콜백 함수 포인터
    void(*OnRank)(int size, StovePCRank* rank, unsigned int rankTotalCount);
};

#endif //STOVEPCDEFINE_H_
