﻿/*! @mainpage StovePCSDK
*/

/**
* @file StovePCSDK.h 
* @brief StovePCSDK 헤더파일
* @details StovePCSDK에서 제공하는 함수를 선언합니다.
*/
#ifndef STOVEPCSDK_H_
#define STOVEPCSDK_H_

#include "StovePCDefine.h"

#ifdef STOVEPCSDK_EXPORTS
#define STOVEPCSDK_API __declspec(dllexport)
#else
#define STOVEPCSDK_API __declspec(dllimport)
#endif



extern "C"
{
    /*!
    *  @fn StovePCResult StovePC_Init(const StovePCConfig& config, const StovePCCallback& callback)
    *  @brief StovePCSDK를 초기화 합니다.
    *  @param[in] StovePCConfig StovePCSDK를 초기화를 위한 설정 정보
    *  @param[in] StovePCCallback StovePCSDK 함수 호출 후 응답받는 콜백 구조체
    *  @return enum StovePCResult 함수 호출 결과   
    */
    STOVEPCSDK_API StovePCResult StovePC_Init(const StovePCConfig& config, const StovePCCallback& callback);

    /*!
    *  @fn StovePCResult StovePC_UnInit()
    *  @brief StovePCSDK의 자원을 해제하고, 종료합니다. 
    *  @return enum StovePCResult 함수 호출 결과   
    */
    STOVEPCSDK_API StovePCResult StovePC_UnInit();

    /*!
    *  @fn StovePCResult StovePC_GetInitState()
    *  @brief StovePC_Init을 호출한 후의 PC SDK 내부 상태를 반환합니다.
    *  @return enum StovePCInitState PC SDK의 초기화 상태 값
    */
    STOVEPCSDK_API StovePCInitState StovePC_GetInitState();


    /*!
    *  @fn StovePCResult StovePC_RunCallback()
    *  @brief 등록한 콜백을 호출합니다.
    *  @return enum StovePCResult 함수 호출 결과   
    */
    STOVEPCSDK_API StovePCResult StovePC_RunCallback();

    /*!
    *  @fn StovePCResult StovePC_GetToken();
    *  @brief 로그인한 사용자의 토큰을 조회합니다. 함수 호출이 성공하면 OnToken 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과   
    */
    STOVEPCSDK_API StovePCResult StovePC_GetToken();

    /*!
    *  @fn StovePCResult StovePC_GetUser();
    *  @brief 로그인한 사용자의 정보를 조회합니다. 함수 호출이 성공하면 OnUser 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetUser();

    /*!
    *  @fn StovePCResult StovePC_GetOwnership();
    *  @brief StovePCConfig에 설정한 게임 아이디의 소유권을 조회합니다. 함수 호출이 성공하면 OnOwnership 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetOwnership();

    /*!
    *  @fn StovePCResult StovePC_GetStat(const char* statId);
    *  @brief 로그인한 사용자의 스탯을 조회합니다. 함수 호출이 성공하면 OnStat 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetStat(const char* statId);

    /*!
    *  @fn StovePCResult StovePC_SetStat(const char* statId, const int statValue);
    *  @brief 로그인한 사용자의 스탯을 설정합니다. 함수 호출이 성공하면 OnSetStat 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_SetStat(const char* statId, const int statValue);

    /*!
    *  @fn StovePCResult StovePC_GetAchievement(const char* achievementId);
    *  @brief 로그인한 사용자의 단일 업적을 조회합니다. 함수 호출이 성공하면 OnAchievement 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetAchievement(const char* achievementId);

    /*!
    *  @fn StovePCResult StovePC_GetAllAchievement();
    *  @brief 로그인한 사용자의 전체 업적을 조회합니다. 함수 호출이 성공하면 OnAllAchievement 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetAllAchievement();

    /*!
    *  @fn StovePCResult StovePCResult StovePC_GetRank(const char* leaderboardId, const unsigned int pageIndex, const unsigned int pageSize, bool includeMyRank);
    *  @brief 특정 페이지의 랭킹을 조회합니다. 함수 호출이 성공하면 OnRank 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetRank(const char* leaderboardId, const unsigned int pageIndex, const unsigned int pageSize, bool includeMyRank);
}

#endif //STOVEPCSDK_H_
