// Fill out your copyright notice in the Description page of Project Settings.


#include "StoveSDKObject.h"
#include "RunTime/CoreUObject/Public/UObject/UObjectIterator.h"
#include "EngineUtils.h"
#include <string>


void OnError(const StovePCError error)
{
    FindStoveSDKObject()->OnError(FStoveError{ static_cast<int>(error.functionType), static_cast<int>(error.result), FString(error.message), error.externalError });
}

void OnInitComplete()
{
    FindStoveSDKObject()->OnInitComplete();
}

void OnToken(const StovePCToken token)
{
    FindStoveSDKObject()->OnToken(FStoveToken{ FString(token.accessToken) });
}

void OnUser(const StovePCUser user)
{
    FStoveUser fUser;
    
    fUser.memberNo = user.memberNo;
    fUser.nickname = FString(WCHAR_TO_TCHAR(user.nickname));
    fUser.gameUserId = FString(user.gameUserId);

    FindStoveSDKObject()->OnUser(fUser);
}

void OnOwnership(int size, StovePCOwnership* ownership)
{
    StovePCOwnership* data = ownership;
    FStoveOwnership* fOwnership = new FStoveOwnership[size];
    for (int index = 0; index < size; index++, data++)
    {
        fOwnership[index].gameId = FString(data->gameId);
        fOwnership[index].gameCode = data->gameCode;
        fOwnership[index].ownershipCode = data->ownershipCode;
        fOwnership[index].purchaseDate = data->purchaseDate;
        fOwnership[index].memberNo = data->memberNo;
    }

    FindStoveSDKObject()->OnOwnership(size, fOwnership);
}


UStoveSDKObject::UStoveSDKObject()
    : _runcallbackResult({ STOVE_PC_NO_ERROR })
    , _isOnCallback(false)
{

}

void UStoveSDKObject::Tick(float DeltaTime)
{
     if (_isOnCallback)
    {
        StovePCResult result = FModuleManager::Get().GetModuleChecked<FStoveSDKPluginModule>("StoveSDKPlugin").RunCallback();
        _runcallbackResult = FStoveResult{ result };
    }
}


bool UStoveSDKObject::IsTickable() const
{
    return true; 
}

TStatId UStoveSDKObject::GetStatId() const
{
    return TStatId();
}



FStoveResult UStoveSDKObject::StoveSDKInit(FStoveConfig fConfig)
{
    StovePCConfig stoveConfig;
    memset(&stoveConfig, 0, sizeof(StovePCConfig));

    std::string env = StringCast<ANSICHAR>(*fConfig.env).Get();
    std::string appKey = StringCast<ANSICHAR>(*fConfig.appKey).Get();
    std::string scret = StringCast<ANSICHAR>(*fConfig.secretKey).Get();
    std::wstring gameid = StringCast<WIDECHAR>(*fConfig.gameId).Get();
    std::wstring logPath = StringCast<WIDECHAR>(*fConfig.logPath).Get();

    stoveConfig.env = const_cast<char*>(env.c_str());
    stoveConfig.appKey = const_cast<char*>(appKey.c_str());
    stoveConfig.secretKey = const_cast<char*>(scret.c_str());
    stoveConfig.gameId = const_cast<wchar_t*>(gameid.c_str());
    stoveConfig.logLevel = StovePCLogLevel(fConfig.logLevel);
    stoveConfig.logPath = const_cast<wchar_t*>(logPath.c_str());

    StovePCCallback stoveCallback{ ::OnError, ::OnInitComplete, ::OnToken, ::OnUser, ::OnOwnership };
    StovePCResult result = FModuleManager::Get().GetModuleChecked<FStoveSDKPluginModule>("StoveSDKPlugin").Init(stoveConfig, stoveCallback);
    if (result == STOVE_PC_NO_ERROR)
    {
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_RunCallback --> Start"));
        _isOnCallback = true; 
    }
    else
    {
        _isOnCallback = false; 
    }
    
    FStoveResult fResult{ result };
    return fResult;
}

FStoveState UStoveSDKObject::StoveSDKGetInitState()
{
    StovePCInitState result = FModuleManager::Get().GetModuleChecked<FStoveSDKPluginModule>("StoveSDKPlugin").InitState();
    FStoveState fResult{ result };
    return fResult;
}


FStoveResult UStoveSDKObject::StoveSDKGetToken()
{
    StovePCResult result = FModuleManager::Get().GetModuleChecked<FStoveSDKPluginModule>("StoveSDKPlugin").GetToken();
    FStoveResult fResult{ result };
    return fResult;
}

FStoveResult UStoveSDKObject::StoveSDKGetUser()
{
    StovePCResult result = FModuleManager::Get().GetModuleChecked<FStoveSDKPluginModule>("StoveSDKPlugin").GetUser();
    FStoveResult fResult{ result };
    return fResult;
}

FStoveResult UStoveSDKObject::StoveSDKGetOwnership()
{
    StovePCResult result = FModuleManager::Get().GetModuleChecked<FStoveSDKPluginModule>("StoveSDKPlugin").GetOwnership();
    FStoveResult fResult{ result };
    return fResult;
}

FStoveResult UStoveSDKObject::StoveSDKUnInit()
{
    StovePCResult result = FModuleManager::Get().GetModuleChecked<FStoveSDKPluginModule>("StoveSDKPlugin").UnInit();
    if (result == STOVE_PC_NO_ERROR)
    {
        _isOnCallback = false;
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("StovePC_RunCallback --> Stop"));
    }
    FStoveResult fResult{ result };
    return fResult;
}